@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2012-2017 Maarten Baert <maarten-baert@hotmail.com>
+Copyright (c) 2012-2020 Maarten Baert <maarten-baert@hotmail.com>
 
 This file is part of SimpleScreenRecorder.
 
@@ -68,10 +68,10 @@ int64_t VideoSource::CalculateNextVideoTimestamp() {
 	return SINK_TIMESTAMP_NONE;
 }
 
-void VideoSource::PushVideoFrame(unsigned int width, unsigned int height, const uint8_t* data, int stride, AVPixelFormat format, int64_t timestamp) {
+void VideoSource::PushVideoFrame(unsigned int width, unsigned int height, const uint8_t* data, int stride, AVPixelFormat format, int colorspace, int64_t timestamp) {
 	SharedLock lock(&m_shared_data);
 	for(SinkData &s : lock->m_sinks) {
-		static_cast<VideoSink*>(s.sink)->ReadVideoFrame(width, height, data, stride, format, timestamp);
+		static_cast<VideoSink*>(s.sink)->ReadVideoFrame(width, height, data, stride, format, colorspace, timestamp);
 	}
 }
 
