@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2012-2017 Maarten Baert <maarten-baert@hotmail.com>
+Copyright (c) 2012-2020 Maarten Baert <maarten-baert@hotmail.com>
 
 This file is part of SimpleScreenRecorder.
 
@@ -19,7 +19,8 @@ along with SimpleScreenRecorder.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "MainWindow.h"
 
-#include "Main.h"
+#include "Logger.h"
+#include "CommandLineOptions.h"
 #include "Icons.h"
 #include "Dialogs.h"
 #include "EnumStrings.h"
@@ -62,10 +63,20 @@ MainWindow::MainWindow()
 	m_stacked_layout->addWidget(m_page_output);
 	m_stacked_layout->addWidget(m_page_record);
 	m_stacked_layout->addWidget(m_page_done);
-	m_stacked_layout->setCurrentWidget(m_page_welcome);
 
 	LoadSettings();
 
+	GoPageStart();
+
+	// warning for non-X11 window systems (e.g. Wayland)
+	if(!IsPlatformX11()) {
+		MessageBox(QMessageBox::Warning, NULL, MainWindow::WINDOW_CAPTION,
+				   MainWindow::tr("You are using a non-X11 window system (e.g. Wayland) which is currently not supported by SimpleScreenRecorder. "
+								  "Several features will most likely not work properly. "
+								  "In order to solve this, you should log out, choose a X11/Xorg session at the login screen, and then log back in."),
+				   BUTTON_OK, BUTTON_OK);
+	}
+
 	// warning for glitch with proprietary NVIDIA drivers
 	if(GetNVidiaDisableFlipping() == NVIDIA_DISABLE_FLIPPING_ASK || GetNVidiaDisableFlipping() == NVIDIA_DISABLE_FLIPPING_YES) {
 		if(NVidiaGetFlipping()) {
@@ -100,27 +111,46 @@ MainWindow::MainWindow()
 
 	// change minimum size based on screen resolution
 	QSize preferred_size = minimumSizeHint() + QSize(style()->pixelMetric(QStyle::PM_ScrollBarExtent), 0);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+	QSize available_size(0, 0);
+	for(QScreen *screen : QApplication::screens()) {
+		QSize size = screen->availableGeometry().size() - QSize(80, 80);
+		available_size = (available_size.isNull())? size : available_size.boundedTo(size);
+	}
+#else
 	QSize available_size = QApplication::desktop()->availableGeometry().size() - QSize(80, 80);
+#endif
 	//qDebug() << preferred_size << available_size;
-	setMinimumSize(preferred_size.boundedTo(available_size));
+	if(!available_size.isNull())
+		setMinimumSize(preferred_size.boundedTo(available_size));
 
-	// maybe show the window
-	if(!g_option_start_hidden)
+	// show the window if needed
+	if(!CommandLineOptions::GetStartHidden()) {
 		show();
+	}
 	m_page_record->UpdateShowHide();
 
+	// start recording and/or activate schedule if needed
+	if(CommandLineOptions::GetStartRecording()) {
+		m_page_record->OnRecordStart();
+	}
+	if(CommandLineOptions::GetActivateSchedule()) {
+		m_page_record->OnScheduleActivate();
+	}
+
 }
 
 MainWindow::~MainWindow() {
-
+	// nothing
 }
 
 void MainWindow::LoadSettings() {
 
-	QSettings settings(GetApplicationUserDir() + "/settings.conf", QSettings::IniFormat);
+	QSettings settings(CommandLineOptions::GetSettingsFile(), QSettings::IniFormat);
 
 	SetNVidiaDisableFlipping(StringToEnum(settings.value("global/nvidia_disable_flipping", QString()).toString(), NVIDIA_DISABLE_FLIPPING_ASK));
 
+	m_page_welcome->LoadSettings(&settings);
 	m_page_input->LoadSettings(&settings);
 	m_page_output->LoadSettings(&settings);
 	m_page_record->LoadSettings(&settings);
@@ -129,19 +159,36 @@ void MainWindow::LoadSettings() {
 
 void MainWindow::SaveSettings() {
 
-	QSettings settings(GetApplicationUserDir() + "/settings.conf", QSettings::IniFormat);
+	QSettings settings(CommandLineOptions::GetSettingsFile(), QSettings::IniFormat);
 	settings.clear();
 
 	settings.setValue("global/nvidia_disable_flipping", EnumToString(GetNVidiaDisableFlipping()));
 
+	m_page_welcome->SaveSettings(&settings);
 	m_page_input->SaveSettings(&settings);
 	m_page_output->SaveSettings(&settings);
 	m_page_record->SaveSettings(&settings);
 
 }
 
+bool MainWindow::IsBusy() {
+	return (QApplication::activeModalWidget() != NULL || QApplication::activePopupWidget() != NULL);
+}
+
 bool MainWindow::Validate() {
-	return m_page_output->Validate();
+	if(!m_page_input->Validate())
+		return false;
+	if(!m_page_output->Validate())
+		return false;
+	return true;
+}
+
+void MainWindow::Quit() {
+	SaveSettings();
+	if(m_nvidia_reenable_flipping) {
+		NVidiaSetFlipping(true);
+	}
+	QApplication::quit();
 }
 
 void MainWindow::closeEvent(QCloseEvent* event) {
@@ -149,14 +196,17 @@ void MainWindow::closeEvent(QCloseEvent* event) {
 		event->ignore();
 		return;
 	}
-	SaveSettings();
-	if(m_nvidia_reenable_flipping) {
-		NVidiaSetFlipping(true);
-	}
 	event->accept();
-	QApplication::quit();
+	Quit();
 }
 
+void MainWindow::GoPageStart() {
+	if(m_page_welcome->GetSkipPage()) {
+		m_stacked_layout->setCurrentWidget(m_page_input);
+	} else {
+		m_stacked_layout->setCurrentWidget(m_page_welcome);
+	}
+}
 void MainWindow::GoPageWelcome() {
 	m_stacked_layout->setCurrentWidget(m_page_welcome);
 }
@@ -165,7 +215,7 @@ void MainWindow::GoPageInput() {
 }
 void MainWindow::GoPageOutput() {
 	m_stacked_layout->setCurrentWidget(m_page_output);
-	m_page_output->PageStart();
+	m_page_output->StartPage();
 }
 void MainWindow::GoPageRecord() {
 	m_stacked_layout->setCurrentWidget(m_page_record);
@@ -175,20 +225,37 @@ void MainWindow::GoPageDone() {
 	m_stacked_layout->setCurrentWidget(m_page_done);
 }
 
-void MainWindow::OnShowHide() {
-	if(isVisible()) {
-		m_old_geometry = geometry();
-		hide();
-	} else {
-		show();
-		if(!m_old_geometry.isNull()) {
-			setGeometry(m_old_geometry);
-			m_old_geometry = QRect();
-		}
+void MainWindow::OnShow() {
+	if(IsBusy())
+		return;
+	if(isVisible())
+		return;
+	show();
+	if(!m_old_geometry.isNull()) {
+		setGeometry(m_old_geometry);
+		m_old_geometry = QRect();
 	}
 	m_page_record->UpdateShowHide();
 }
 
+void MainWindow::OnHide() {
+	if(IsBusy())
+		return;
+	if(!isVisible())
+		return;
+	m_old_geometry = geometry();
+	hide();
+	m_page_record->UpdateShowHide();
+}
+
+void MainWindow::OnShowHide() {
+	if(isVisible()) {
+		OnHide();
+	} else {
+		OnShow();
+	}
+}
+
 void MainWindow::OnSysTrayActivated(QSystemTrayIcon::ActivationReason reason) {
 	if(reason == QSystemTrayIcon::Trigger || reason == QSystemTrayIcon::DoubleClick) {
 		OnShowHide();
