@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2012-2017 Maarten Baert <maarten-baert@hotmail.com>
+Copyright (c) 2012-2020 Maarten Baert <maarten-baert@hotmail.com>
 
 This file is part of SimpleScreenRecorder.
 
@@ -26,6 +26,7 @@ along with SimpleScreenRecorder.  If not, see <http://www.gnu.org/licenses/>.
 #include "OutputManager.h"
 #include "ElidedLabel.h"
 #include "HotkeyListener.h"
+#include "DialogRecordSchedule.h"
 
 class MainWindow;
 
@@ -38,6 +39,9 @@ class X11Input;
 class GLInjectLauncher;
 class GLInjectInput;
 #endif
+#if SSR_USE_V4L2
+class V4L2Input;
+#endif
 #if SSR_USE_ALSA
 class ALSAInput;
 #endif
@@ -57,7 +61,7 @@ class PageRecord : public QWidget {
 	Q_OBJECT
 
 private:
-	static const int PRIORITY_RECORD, PRIORITY_PREVIEW;
+	static constexpr int PRIORITY_RECORD = 0, PRIORITY_PREVIEW = -1;
 
 private:
 	MainWindow *m_main_window;
@@ -65,7 +69,16 @@ private:
 	bool m_page_started, m_input_started, m_output_started, m_previewing;
 	bool m_recorded_something, m_wait_saving, m_error_occurred;
 
+	bool m_schedule_active;
+	unsigned int m_schedule_position;
+	enum_schedule_time_zone m_schedule_time_zone;
+	std::vector<ScheduleEntry> m_schedule_entries;
+
 	PageInput::enum_video_area m_video_area;
+	bool m_video_area_follow_fullscreen;
+#if SSR_USE_V4L2
+	QString m_v4l2_device;
+#endif
 	unsigned int m_video_x, m_video_y, m_video_in_width, m_video_in_height;
 	unsigned int m_video_frame_rate;
 	bool m_video_scaling;
@@ -92,6 +105,9 @@ private:
 #if SSR_USE_OPENGL_RECORDING
 	std::unique_ptr<GLInjectInput> m_gl_inject_input;
 #endif
+#if SSR_USE_V4L2
+	std::unique_ptr<V4L2Input> m_v4l2_input;
+#endif
 #if SSR_USE_ALSA
 	std::unique_ptr<ALSAInput> m_alsa_input;
 #endif
@@ -109,7 +125,9 @@ private:
 
 	HotkeyCallback m_hotkey_start_pause;
 
-	QPushButton *m_pushbutton_start_pause;
+	QPushButton *m_pushbutton_record;
+	QLabel *m_label_schedule_status;
+	QPushButton *m_pushbutton_schedule_activate, *m_pushbutton_schedule_edit;
 
 	QCheckBox *m_checkbox_hotkey_enable;
 #if SSR_USE_ALSA
@@ -121,6 +139,7 @@ private:
 	QLabel *m_label_info_total_time, *m_label_info_frame_rate_in, *m_label_info_frame_rate_out, *m_label_info_size_in, *m_label_info_size_out;
 	ElidedLabel *m_label_info_file_name;
 	QLabel *m_label_info_file_size, *m_label_info_bit_rate;
+	QCheckBox *m_checkbox_show_recording_area;
 
 	QStackedLayout *m_stacked_layout_preview;
 	QWidget *m_preview_page1, *m_preview_page2;
@@ -136,7 +155,13 @@ private:
 	QAction *m_systray_action_start_pause, *m_systray_action_cancel, *m_systray_action_save;
 	QAction *m_systray_action_show_hide, *m_systray_action_quit;
 
-	QTimer *m_timer_update_info;
+	QSocketNotifier *m_stdin_notifier;
+	QByteArray m_stdin_buffer;
+	bool m_stdin_reentrant;
+
+	QTimer *m_timer_schedule, *m_timer_update_info;
+
+	std::unique_ptr<RecordingFrameWindow> m_recording_frame;
 
 public:
 	PageRecord(MainWindow* main_window);
@@ -164,10 +189,15 @@ private:
 	void FinishOutput();
 	void UpdateInput();
 	void UpdateSysTray();
-	void UpdateRecordPauseButton();
+	void UpdateRecordButton();
+	void UpdateSchedule();
 	void UpdatePreview();
 
+	QString ReadStdinCommand();
+
 public:
+	inline enum_schedule_time_zone GetScheduleTimeZone() { return m_schedule_time_zone; }
+	inline std::vector<ScheduleEntry> GetScheduleEntries() { return m_schedule_entries; }
 	inline bool IsHotkeyEnabled() { return m_checkbox_hotkey_enable->isChecked(); }
 	inline bool IsHotkeyCtrlEnabled() { return m_checkbox_hotkey_ctrl->isChecked(); }
 	inline bool IsHotkeyShiftEnabled() { return m_checkbox_hotkey_shift->isChecked(); }
@@ -177,8 +207,11 @@ public:
 #if SSR_USE_ALSA
 	inline bool AreSoundNotificationsEnabled() { return m_checkbox_sound_notifications_enable->isChecked(); }
 #endif
+	inline bool GetShowRecordingArea() { return m_checkbox_show_recording_area->isChecked(); }
 	inline unsigned int GetPreviewFrameRate() { return m_spinbox_preview_frame_rate->value(); }
 
+	inline void SetScheduleTimeZone(enum_schedule_time_zone time_zone) { m_schedule_time_zone = (enum_schedule_time_zone) clamp((unsigned int) time_zone, 0u, (unsigned int) SCHEDULE_TIME_ZONE_COUNT - 1); }
+	inline void SetScheduleEntries(const std::vector<ScheduleEntry>& schedule) { m_schedule_entries = schedule; }
 	inline void SetHotkeyEnabled(bool enable) { m_checkbox_hotkey_enable->setChecked(enable); }
 	inline void SetHotkeyCtrlEnabled(bool enable) { m_checkbox_hotkey_ctrl->setChecked(enable); }
 	inline void SetHotkeyShiftEnabled(bool enable) { m_checkbox_hotkey_shift->setChecked(enable); }
@@ -188,6 +221,7 @@ public:
 #if SSR_USE_ALSA
 	inline void SetSoundNotificationsEnabled(bool enable) { m_checkbox_sound_notifications_enable->setChecked(enable); }
 #endif
+	inline void SetShowRecordingArea(bool enable) { m_checkbox_show_recording_area->setChecked(enable); }
 	inline void SetPreviewFrameRate(unsigned int frame_rate) { m_spinbox_preview_frame_rate->setValue(frame_rate); }
 
 public slots:
@@ -196,13 +230,23 @@ public slots:
 #if SSR_USE_ALSA
 	void OnUpdateSoundNotifications();
 #endif
+	void OnUpdateRecordingFrame();
+
+public slots:
+	void OnRecordStart();
+	void OnRecordPause();
+	void OnRecordStartPause();
+	void OnRecordCancel(bool confirm = true);
+	void OnRecordSave(bool confirm = true);
+	void OnScheduleTimer();
+	void OnScheduleActivate();
+	void OnScheduleDeactivate();
+	void OnScheduleActivateDeactivate();
+	void OnScheduleEdit();
+	void OnPreviewStartStop();
 
 private slots:
-	void OnRecordStartPause();
-	void OnPreviewStartStop();
-	void OnCancel();
-	void OnSave();
-
+	void OnStdin();
 	void OnUpdateInformation();
 	void OnNewLogLine(Logger::enum_type type, QString string);
 
