@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2012-2017 Maarten Baert <maarten-baert@hotmail.com>
+Copyright (c) 2012-2020 Maarten Baert <maarten-baert@hotmail.com>
 
 This file is part of SimpleScreenRecorder.
 
@@ -19,8 +19,8 @@ along with SimpleScreenRecorder.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "PageWelcome.h"
 
+#include "CommandLineOptions.h"
 #include "Icons.h"
-#include "Main.h"
 #include "MainWindow.h"
 
 PageWelcome::PageWelcome(MainWindow* main_window)
@@ -36,18 +36,21 @@ PageWelcome::PageWelcome(MainWindow* main_window)
 	{
 		QLabel *label_logo = new QLabel(scrollarea_contents);
 		label_logo->setPixmap(QPixmap(":/header.png"));
+		label_logo->setScaledContents(true);
 		QLabel *label_welcome = new QLabel(scrollarea_contents);
 		label_welcome->setText(tr("<p>Welcome to SimpleScreenRecorder!</p>\n\n"
 								  "<p>Despite the name, this program actually has a lot of options. Don't worry though, there are really just two things that you "
 								  "need to know. One, the default settings are usually fine. If you don't know what something does, just use the default. "
 								  "Two, almost all settings have tooltips. Just hover the mouse over something to find out what it does.</p>\n\n"
 								  "<p>For more information:<br>\n"
-								  "%1</p>").arg("<a href=\"http://www.maartenbaert.be/simplescreenrecorder/\">http://www.maartenbaert.be/simplescreenrecorder/</a>"));
+								  "%1</p>").arg("<a href=\"https://www.maartenbaert.be/simplescreenrecorder/\">https://www.maartenbaert.be/simplescreenrecorder/</a>"));
 		label_welcome->setWordWrap(true);
 		label_welcome->setTextFormat(Qt::RichText);
 		label_welcome->setTextInteractionFlags(Qt::TextBrowserInteraction);
 		label_welcome->setOpenExternalLinks(true);
 		QPushButton *button_about = new QPushButton(tr("About SimpleScreenRecorder"), scrollarea_contents);
+		m_checkbox_skip_page = new QCheckBox(tr("Skip this page next time"), scrollarea_contents);
+		m_checkbox_skip_page->setToolTip(tr("Go directly to the input page when the program is started."));
 
 		connect(button_about, SIGNAL(clicked()), this, SLOT(AboutDialog()));
 
@@ -67,6 +70,7 @@ PageWelcome::PageWelcome(MainWindow* main_window)
 			layout2->addWidget(button_about);
 			layout2->addStretch();
 		}
+		layout->addWidget(m_checkbox_skip_page);
 		layout->addStretch();
 	}
 	QPushButton *button_continue = new QPushButton(g_icon_go_next, tr("Continue"), this);
@@ -87,6 +91,15 @@ PageWelcome::PageWelcome(MainWindow* main_window)
 
 }
 
+
+void PageWelcome::LoadSettings(QSettings* settings) {
+	SetSkipPage(settings->value("welcome/skip_page", false).toBool());
+}
+
+void PageWelcome::SaveSettings(QSettings* settings) {
+	settings->setValue("welcome/skip_page", GetSkipPage());
+}
+
 void PageWelcome::AboutDialog() {
 	DialogAbout dialog(this);
 	dialog.exec();
